<?php
/************************************************************************/
#  TeamSpeak Block For PHP-NUKE: Web Portal System
#  ===============================================
# 
# Copyright (c) 2005 by Tsuchida, Butcher and PrideKillersInc =PKI= Ver 1.2
# http://www.PrideKillersInc.com
# January 2005
# - Fixes:
#   1) Combined operability from two blocks allowing user to log into a non-secure TS server by clicking onto the channel name and to log into a secure server by using the pulldown menu.
#   2) Changed standard green bullet to blinking green bullet.
#   3) Sorted main and sub channels by Sort Order (setting in TeamSpeak under "Edit Channel")
#   4) Sorted main and sub channels alphabeticly if they have the same Sort Order (setting in TeamSpeak under "Edit Channel")
#   5) Config flag for displaying the pulldown menu. Set $DisplayPullDown to 0 to remove the pulldown menu from the block display
#
# modified by: Tsuchida, Butcher and PrideKillersInc =PKI=
# http://www.PrideKillersInc.com
# Febuary 2005
# - Fixes:
#   1) Added icons for user status. Now you cna tell when users are AFK, muted, etc.
#   2) Added ability for admin to block anyone logging in anonymously.
#   3) Added ability for user to log in with a name other than the one they are logged into the website with.
#
/************************************************************************/

class tss2info {
// ts2info.php
// (c) 2003 by Christian Mller
// non-commercial use approved, commercial users please contact me at mueller@fmi.uni-passau.de
// i give no guarantees whatsoever about the correct functioning of this script
// use at your own risk
// date: 20/06/03, 23:28

// **** settings - to be edited before first use ****
var $serverURL = "xxx.xxx.xxx.xxx"; // will be used for the connect link, don't use localhost here
var $serverAddress = "xxx.xxx.xxx.xxx"; // can be ip address or url
var $serverQueryPort = 51234; // default 51234, must be accessible and usable. check server.ini
var $serverUDPPort = 8767; // default 8767
// **** end of settings ****

//internal
var $socket;

// external
var $serverStatus = "offline";
var $playerList = array();
var $channelList = array();


// opens a connection to the teamspeak server
function getSocket($host, $port, $errno, $errstr, $timeout) {
  unset($socket);
  $attempts = 1;
  while($attempts <= 1 and !$socket) {
    $attempts++;
    @$socket = fsockopen($host, $port, $errno, $errstr, $timeout);
    $this->errno = $errno;
    $this->errstr = $errstr;
    if($socket and fread($socket, 4) == "[TS]") {
      fgets($socket, 128);
      return $socket;
    }
  }// end while
  return false;
}// end function getSocket(...)

// sends a query to the teamspeak server
function sendQuery($socket, $query) {
  fputs($socket, $query."\n");
}// end function sendQuery(...)

// answer OK?
function getOK($socket) {
  $result = fread($socket, 2);
  fgets($socket, 128);
  return($result == "OK");
}// end function getOK(...)

// closes the connection to the teamspeak server
function closeSocket($socket) {
  fputs($socket, "quit");
  fclose($socket);
}// end function closeSocket(...)

// retrieves the next argument in a tabulator-separated string (PHP scanf function bug workaround)
function getNext($evalString) {
  $pos = strpos($evalString, "\t");
  if(is_integer($pos)) {
    return substr($evalString, 0, $pos);
  } else {
    return $evalString;
  }// end if
}// end function getNext($evalString);

// removes the first argument in a tabulator-separated string (PHP scanf function bug workaround)
function chopNext($evalString) {
  $pos = strpos($evalString, "\t");
  if(is_integer($pos)) {
    return substr($evalString, $pos + 1);
  } else {
    return "";
  }// end if
}// end function chopNext($evalString)

// strips the quotes around a string
function stripQuotes($evalString) {
  if(strpos($evalString, '"') == 0) $evalString = substr($evalString, 1, strlen($evalString) - 1);
  if(strrpos($evalString, '"') == strlen($evalString) - 1) $evalString = substr($evalString, 0, strlen($evalString) - 1);

  return $evalString;
}// end function stripQuotes($evalString)

// returns the codec name
function getVerboseCodec($codec) {
  if($codec == 0) {
    $codec = "CELP 5.1 Kbit";
  } elseif($codec == 1) {
    $codec = "CELP 6.3 Kbit";
  } elseif($codec == 2) {
    $codec = "GSM 14.8 Kbit";
  } elseif($codec == 3) {
    $codec = "GSM 16.4 Kbit";
  } elseif($codec == 4) {
    $codec = "CELP Windows 5.2 Kbit";
  } elseif($codec == 5) {
    $codec = "Speex 3.4 Kbit";
  } elseif($codec == 6) {
    $codec = "Speex 5.2 Kbit";
  } elseif($codec == 7) {
    $codec = "Speex 7.2 Kbit";
  } elseif($codec == 8) {
    $codec = "Speex 9.3 Kbit";
  } elseif($codec == 9) {
    $codec = "Speex 12.3 Kbit";
  } elseif($codec == 10) {
    $codec = "Speex 16.3 Kbit";
  } elseif($codec == 11) {
    $codec = "Speex 19.5 Kbit";
  } elseif($codec == 12) {
    $codec = "Speex 25.9 Kbit";
  } else {
    $codec = "unknown (".$codec.")";
  }// end if
  return $codec;
}// end function getVerboseCodec($codec);

function getInfo() {
// ---=== main program ===---

// establish connection to teamspeak server
$this->socket = $this->getSocket($this->serverAddress, $this->serverQueryPort, $errno, $errstr, 0.3);
if($this->socket == false) {
  return;
  echo ("No Server");
} else {
  $this->serverStatus = "online";

  // select the one and only running server on port 8767
  $this->sendQuery($this->socket, "sel ".$this->serverUDPPort);

  // retrieve answer "OK"
  if(!$this->getOK($this->socket)) {
    echo "Server didn't answer \"OK\" after last command. Aborting.";
    return;
  }// end if

  // retrieve player list
  $this->sendQuery($this->socket,"pl");

  // read player info
  $this->playerList = array();
  do {
    $playerinfo = fscanf($this->socket, "%s %d %d %d %d %d %d %d %d %d %d %d %d %s %s");
    list($playerid, $channelid, $receivedpackets, $receivedbytes, $sentpackets, $sentbytes, $d1, $d2, $totaltime, $idletime, $d3, $d4, $d5, $s1, $playername) = $playerinfo;
    if($playerid != "OK") {
      $this->playerList[$playerid] = array("playerid" => $playerid,
      "channelid" => $channelid,
      "receivedpackets" => $receivedpackets,
      "receivedbytes" => $receivedbytes,
      "sentpackets" => $sentpackets,
      "sentbytes" => $sentbytes,
      "totaltime" => $totaltime,
      "idletime" => $idletime,

      //VTB FOR DEBUG!!!

      "d1" => $d1,
      "d2" => $d2,
      "d3" => $d3,
      "d4" => $d4,
      "d5" => $d5,
      "s1" => $s1,

      "playername" => $this->stripQuotes($playername));
    }// end if
  } while($playerid != "OK");

  // retrieve channel list
  $this->sendQuery($this->socket,"cl");

  // read channel info
  $this->channelList = array();
  do {
    $channelinfo = "";
    do {
      $input = fread($this->socket, 1);
      if($input != "\n" && $input != "\r")
      {
//      echo "input $input<br>";
       $channelinfo .= $input;
      }
    } while($input != "\n");

    $channelid = $this->getNext($channelinfo);
     $channelinfo = $this->chopNext($channelinfo);
    $codec = $this->getNext($channelinfo);
     $channelinfo = $this->chopNext($channelinfo);
    $parent = $this->getNext($channelinfo);
     $channelinfo = $this->chopNext($channelinfo);
    $order = $this->getNext($channelinfo);
     $channelinfo = $this->chopNext($channelinfo);
    $maxplayers = $this->getNext($channelinfo);
     $channelinfo = $this->chopNext($channelinfo);
    $channelname = $this->getNext($channelinfo);
     $channelinfo = $this->chopNext($channelinfo);
    $d2 = $this->getNext($channelinfo);
     $channelinfo = $this->chopNext($channelinfo);
    $d3 = $this->getNext($channelinfo);
     $channelinfo = $this->chopNext($channelinfo);
    $topic = $this->getNext($channelinfo);

    if($channelid != "OK") {
      if($isdefault == "Default") $isdefault = 1; else $isdefault = 0;
      
      // determine number of players in channel
      $playercount = 0;
      foreach($this->playerList as $playerInfo) {
        if($playerInfo[channelid] == $channelid) $playercount++;
      }// end foreach

      $this->channelList[$channelid] = array("channelid" => $channelid,
      "codec" => $codec,
      "parent" => $parent,
      "maxplayers" => $maxplayers,
      "channelname" => $this->stripQuotes($channelname),
      "isdefault" => $isdefault,
      "topic" => $this->stripQuotes($topic),
      "currentplayers" => $playercount,
      "order" => $order,               //VTB TEST
      "d2" => $d2,               //VTB TEST
      "d3" => $d3);              //VTB TEST
   }// end if
  } while($channelid != "OK");

  // close connection to teamspeak server
  $this->closeSocket($this->socket);

  }// end getInfo()
}// class tss2info
}
$tss2info = new tss2info;
?>